@@ -2,7 +2,7 @@
 
 # IE2G    Converts Microsoft Internet Explorer favorites to Galeon bookmarks
 # 
-# Version:   0.21  (2002-01-27)
+# Version:   0.22  (2002-02-28)
 #
 # Copyright  (C) 2001-2002 Francesco Poli, <e_frx@inwind.it>
 #                          Via Amici 19 50131 Firenze, Italy
@@ -43,7 +43,7 @@ if (@ARGV == 1 and $ARGV[0] eq "--help")
     "Option VERSION specifies target Galeon version: it must be 0.8.2 or",
     " 0.12\n\n",
     "With the option --help, display this help and exit.\n\n",
-    "IE2G version 0.21, Copyright (C) 2001-2002 Francesco Poli\n",
+    "IE2G version 0.22, Copyright (C) 2001-2002 Francesco Poli\n",
     "IE2G comes with ABSOLUTELY NO WARRANTY.\n",
     "This is free software, and you are welcome to redistribute it\n",
     "under certain conditions; read file `COPYING' for details.\n\n",
@@ -73,19 +73,40 @@ SWITCH:
 }
 
 # discard final slash, if present
-$dest =~ s/\/$//;
+$source =~ s/\/$//;
+$dest   =~ s/\/$//;
 
 # open output file
 open XML, ">$dest/bookmarks.ie2g"  or die
 "ie2g.pl: cannot write $dest/bookmarks.ie2g\n";
 
-# start recursion
+# initialize element counter
 $id = 0;
-&create_category($source, 1, $indent, $target);
+
+# initialize warning array
+@w_book_not_valid = ();
+
+# start recursion
+&create_category($source, 1, $indent, $target, "");
 
 # close output file
 close XML;
 
+# write warnings to stderr
+if (@w_book_not_valid)
+{
+    warn
+    "WARNING: the following files don't seem to be valid IE favorite files:\n";
+
+    foreach my $not_valid(@w_book_not_valid)
+    {
+        warn "$not_valid\n";
+    }
+
+    warn
+    "Since their URLs were unavailable, fictitious URLs have been written\n"
+}
+
 exit;
 
 
@@ -96,16 +117,22 @@ sub  create_category
 {
 
     # get parameters
-    my ($dir, $header, $indent, $target) = @_;
+    my ($dir, $header, $indent, $target, $wherewasi) = @_;
 
     # change working directory
     chdir $dir;
 
+    # this will store current directory
+    my $whereami;
+
     # a flag to remember we must finish to write a tag
     my $stmissing = 0;
 
     if ($header == 1)
     {
+        # where am I now?
+        $whereami = $dir;
+
         # report activity on stdout
         print "Starting bookmark conversion...\n\n";
 
@@ -122,6 +149,9 @@ sub  create_category
     }
     else
     {
+        # where am I now?
+        $whereami = $wherewasi."/".$dir;
+
         # report activity on stdout
         print $indent."opening category \"".$dir."\"\n";
 
@@ -195,7 +225,7 @@ sub  create_category
     # create sites
     foreach my $site(@sites)
     {
-        &create_site($site, "  $indent", $target);
+        &create_site($site, "  $indent", $target, $whereami);
     }
 
     # create subcategories
@@ -203,9 +233,9 @@ sub  create_category
     {
         # discard newline and slash
         chomp $subdir;
-        chop  $subdir;      
+        $subdir =~ s/\/$//;
 
-        &create_category($subdir, 0, "  $indent", $target);
+        &create_category($subdir, 0, "  $indent", $target, $whereami);
     }
 
     if ($header == 1)
@@ -255,7 +285,7 @@ sub  create_site
 {
 
     # get parameters
-    my ($filename, $indent, $target) = @_;
+    my ($filename, $indent, $target, $whereami) = @_;
 
     # discard newline
     chomp $filename;
@@ -264,11 +294,38 @@ sub  create_site
     open BOOK, "<$filename"  or 
     ( print $indent."WARNING: cannot read \"$filename\"\n" and return );
 
-    # extract the URL
-    do { $line = <BOOK>; } until $line =~ /^\[InternetShortcut\]/;
-    do { $line = <BOOK>; } until $line =~ /^URL=(.*)$/;
-    $url = $1;
-    $url =~ s/\r$//;
+    # extract the URL, if possible
+    my $url;
+    my $is_OK = 0;
+    CHECK: while (my $line = <BOOK>)
+    {
+        if ($line =~ /^\[InternetShortcut\]/)
+        {
+            while ($line = <BOOK>)
+            {
+                if ($line =~ /^URL=(.*)$/)
+                {
+                    $is_OK = 1;
+                    $url = $1;
+                    last CHECK;
+                }
+            }
+        }
+    }
+
+    if ($is_OK)
+    {
+        # analysed file seems OK
+        $url =~ s/\r$//;
+    }
+    else
+    {
+        # analysed file doesn't seem to be a valid IE favorite file
+        push @w_book_not_valid, $whereami."/".$filename;
+
+        # setup a fictitious URL
+        $url = "this is not a URL";
+    }
 
     # discard ".url"
     $filename =~ s/\.url$//;
